@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileCopy.cc,v 1.96.2.2 2003/12/22 13:18:50 lav Exp $ */
+/* $Id: FileCopy.cc,v 1.102 2004/03/19 14:20:55 lav Exp $ */
 
 /* FileCopyPeer behaviour:
     1) when suspended, does nothing
@@ -124,6 +124,8 @@ int FileCopy::Do()
       && put->GetSeekPos()>=get->GetSize())
       {
 	 debug((9,_("copy: destination file is already complete\n")));
+	 if(get->GetDate()!=NO_DATE)
+	    goto pre_CONFIRM_WAIT;  // have to set the date.
 	 goto pre_GET_DONE_WAIT;
       }
       if(!put->IOReady())
@@ -246,20 +248,7 @@ int FileCopy::Do()
       if(b==0) // eof
       {
 	 debug((10,"copy: get hit eof\n"));
-      eof:
-	 put->Resume();
-	 if(line_buffer)
-	 {
-	    line_buffer->Get(&b,&s);
-	    put->Put(b,s);
-	    line_buffer->Skip(s);
-	 }
-	 put->SetDate(get->GetDate());
-	 put->PutEOF();
-	 get->Suspend();
-	 put_eof_pos=put->GetRealPos();
-	 state=CONFIRM_WAIT;
-	 return MOVED;
+	 goto eof;
       }
 
       rate_add=put_buf;
@@ -323,6 +312,25 @@ int FileCopy::Do()
       }
       return m;
    }
+
+   eof:
+      if(line_buffer)
+      {
+	 line_buffer->Get(&b,&s);
+	 put->Put(b,s);
+	 line_buffer->Skip(s);
+      }
+   pre_CONFIRM_WAIT:
+      put->SetDate(get->GetDate());
+      if(get->GetSize()!=NO_SIZE && get->GetSize()!=NO_SIZE_YET)
+	 put->SetEntitySize(get->GetSize());
+      put->PutEOF();
+      get->Suspend();
+      put->Resume();
+      put_eof_pos=put->GetRealPos();
+      debug((10,"copy: waiting for put confirmation\n"));
+      state=CONFIRM_WAIT;
+      m=MOVED;
    case(CONFIRM_WAIT):
       if(put->Error())
 	 goto put_error;
@@ -414,7 +422,7 @@ FileCopy::~FileCopy()
 {
    Delete(get);
    Delete(put);
-   Delete(line_buffer);
+   delete line_buffer;
    xfree(error_text);
    Delete(rate);
    Delete(rate_for_eta);
@@ -655,9 +663,8 @@ bool FileCopyPeer::Done()
    return false;
 }
 
-FileCopyPeer::FileCopyPeer(direction m)
+FileCopyPeer::FileCopyPeer(dir_t m) : IOBuffer(m)
 {
-   mode=m;
    want_size=false;
    want_date=false;
    start_transfer=true;
@@ -756,6 +763,8 @@ int FileCopyPeerFA::Do()
 	 {
 	    if(date!=NO_DATE && date!=NO_DATE_YET)
 	       session->SetDate(date);
+	    if(e_size!=NO_SIZE && e_size!=NO_SIZE_YET)
+	       session->SetSize(e_size);
 	    res=session->StoreStatus();
 	    if(res==FA::OK)
 	    {
@@ -896,7 +905,8 @@ void FileCopyPeerFA::OpenSession()
 	 debug((10,"copy dst: seek past eof (seek_pos=%lld, size=%lld)\n",
 		  (long long)seek_pos,(long long)e_size));
 	 eof=true;
-	 return;
+	 if(date==NO_DATE || date==NO_DATE_YET)
+	    return;
       }
    }
    session->Open(file,FAmode,seek_pos);
@@ -1189,7 +1199,7 @@ FileCopyPeerFA *FileCopyPeerFA::New(FileAccess *s,const char *url,int m,bool reu
 // FileCopyPeerFDStream
 #undef super
 #define super FileCopyPeer
-FileCopyPeerFDStream::FileCopyPeerFDStream(FDStream *o,direction m)
+FileCopyPeerFDStream::FileCopyPeerFDStream(FDStream *o,dir_t m)
    : FileCopyPeer(m)
 {
    if(o==0 && m==PUT)
@@ -1214,7 +1224,7 @@ FileCopyPeerFDStream::FileCopyPeerFDStream(FDStream *o,direction m)
       write_allowed=false;
    put_ll_timer=0;
    if(m==PUT)
-      put_ll_timer=new Timer(TimeDiff(0,100));
+      put_ll_timer=new Timer(TimeDiff(0,200));
 }
 FileCopyPeerFDStream::~FileCopyPeerFDStream()
 {
@@ -1549,14 +1559,41 @@ int FileCopyPeerFDStream::Put_LL(const char *buf,int len)
 	 return -1;
       }
       if(stream->NonFatalError(errno))
+      {
+	 // in case of full disk, check file correctness.
+	 if(errno==ENOSPC && can_seek)
+	 {
+	    struct stat st;
+	    if(fstat(fd,&st)!=-1)
+	    {
+	       if(st.st_size<seek_base+pos-in_buffer)
+	       {
+		  // workaround solaris nfs bug. It can lose data if disk is full.
+		  if(buffer_ptr>=seek_base+pos-in_buffer-buffer_ptr-st.st_size)
+		     UnSkip(seek_base+pos-in_buffer-st.st_size);
+		  else
+		  {
+		     Empty();
+		     pos=st.st_size;
+		  }
+	       }
+	    }
+	 }
 	 return 0;
+      }
       stream->MakeErrorText();
       SetError(stream->error_text);
       return -1;
    }
    stream->clear_status();
-   if(res==len)
+   if(res==len && skip_cr)
+   {
       res+=skip_cr;
+      // performance gets worse because of writing a single char,
+      // but leaving uncomplete line on screen allows mixing it with debug text.
+      if(write(fd,"\n",1)==1)
+	 res+=1;
+   }
    if(put_ll_timer)
       put_ll_timer->Reset();
    return res;
