@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileSet.cc,v 1.35 2003/06/21 08:40:06 lav Exp $ */
+/* $Id: FileSet.cc,v 1.42 2004/04/22 06:32:54 lav Exp $ */
 
 #include <config.h>
 #include "FileSet.h"
@@ -42,6 +42,11 @@
 #include "IdNameCache.h"
 #include "PatternSet.h"
 
+#define NO_SIZE	     (-1L)
+#define NO_SIZE_YET  (-2L)
+#define NO_DATE	     ((time_t)-1L)
+#define NO_DATE_YET  ((time_t)-2L)
+
 void  FileInfo::Merge(const FileInfo& f)
 {
    if(strcmp(name,f.name))
@@ -152,7 +157,7 @@ FileSet::FileSet(FileSet const *set)
 {
    ind=set->ind;
    fnum=set->fnum;
-   sorted = false;
+   sorted=false;
    if(fnum==0)
       files=files_sort=0;
    else
@@ -222,7 +227,7 @@ void FileSet::Sort(sort_e newsort, bool casefold)
 	 files_sort[i] = files[i];
    }
 
-   sorted = true;
+   sorted=true;
 
    if(casefold) compare = strcasecmp;
    else compare = strcmp;
@@ -242,8 +247,8 @@ void FileSet::Unsort()
 {
    if(!sorted) return;
    xfree(files_sort);
-   files_sort = files;
-   sorted = false;
+   files_sort=files;
+   sorted=false;
 }
 
 void FileSet::Empty()
@@ -285,6 +290,16 @@ void FileSet::SubtractNotIn(const FileSet *set)
       if(!set->FindByName(files[i]->name))
 	 Sub(i--);
 }
+void FileSet::SubtractSameType(const FileSet *set)
+{
+   for(int i=0; i<fnum; i++)
+   {
+      FileInfo *f=set->FindByName(files[i]->name);
+      if(f && files[i]->defined&FileInfo::TYPE && f->defined&FileInfo::TYPE
+      && files[i]->filetype==f->filetype)
+	 Sub(i--);
+   }
+}
 
 void FileSet::SubtractOlderThan(time_t t)
 {
@@ -312,6 +327,18 @@ void FileSet::SubtractDirs()
       }
    }
 }
+void FileSet::SubtractNotDirs()
+{
+   for(int i=0; i<fnum; i++)
+   {
+      if(!(files[i]->defined&FileInfo::TYPE)
+      || files[i]->filetype!=FileInfo::DIRECTORY)
+      {
+	 Sub(i);
+	 i--;
+      }
+   }
+}
 
 void FileSet::ExcludeDots()
 {
@@ -371,11 +398,7 @@ void FileSet::Count(int *d,int *f,int *s,int *o)
 {
    for(int i=0; i<fnum; i++)
    {
-      if(!(files[i]->defined&FileInfo::TYPE))
-      {
-	 if(o) (*o)++;
-      }
-      else switch(files[i]->filetype)
+      switch(files[i]->filetype)
       {
       case(FileInfo::DIRECTORY):
 	 if(d) (*d)++; break;
@@ -383,6 +406,8 @@ void FileSet::Count(int *d,int *f,int *s,int *o)
 	 if(f) (*f)++; break;
       case(FileInfo::SYMLINK):
 	 if(s) (*s)++; break;
+      case(FileInfo::UNKNOWN):
+	 if(o) (*o)++;
       }
    }
 }
@@ -599,12 +624,19 @@ void FileSet::SubtractCurr()
 
 void FileInfo::Init()
 {
+   filetype=UNKNOWN;
+   mode=(mode_t)-1;
+   date=NO_DATE;
+   date_prec=0;
+   size=NO_SIZE;
+   nlinks=0;
    name=0;
    defined=0;
    symlink=0;
    data=0;
    user=0; group=0;
    rank=0;
+   longname=0;
 }
 
 FileInfo::FileInfo(const FileInfo &fi)
@@ -621,6 +653,7 @@ FileInfo::FileInfo(const FileInfo &fi)
    date_prec=fi.date_prec;
    size=fi.size;
    nlinks=fi.nlinks;
+   longname=xstrdup(fi.longname);
 }
 
 #ifndef S_ISLNK
@@ -680,11 +713,167 @@ check_again:
 #endif /* HAVE_LSTAT */
 }
 
+/* parse_ls_line: too common procedure to make it protocol specific */
+/*
+-rwxr-xr-x   1 lav      root         4771 Sep 12  1996 install-sh
+-rw-r--r--   1 lav      root         1349 Feb  2 14:10 lftp.lsm
+drwxr-xr-x   4 lav      root         1024 Feb 22 15:32 lib
+lrwxrwxrwx   1 lav      root           33 Feb 14 17:45 ltconfig -> /usr/share/libtool/ltconfig
+NOTE: group may be missing.
+*/
+FileInfo *FileInfo::parse_ls_line(const char *line_c,const char *tz)
+{
+   char *line=alloca_strdup(line_c);
+   char *next=0;
+   FileInfo *fi=0; /* don't instantiate until we at least have something */
+#define FIRST_TOKEN strtok_r(line," \t",&next)
+#define NEXT_TOKEN  strtok_r(NULL," \t",&next)
+#define ERR do{delete fi;return(0);}while(0)
+
+   /* parse perms */
+   char *t = FIRST_TOKEN;
+   if(t==0)
+      ERR;
+
+   fi = new FileInfo;
+   switch(t[0])
+   {
+   case('l'):  // symlink
+      fi->SetType(fi->SYMLINK);
+      break;
+   case('d'):  // directory
+      fi->SetType(fi->DIRECTORY);
+      break;
+   case('-'):  // plain file
+      fi->SetType(fi->NORMAL);
+      break;
+   case('b'): // block
+   case('c'): // char
+   case('p'): // pipe
+   case('s'): // sock
+   case('D'): // Door
+      return 0;  // ignore
+   default:
+      ERR;
+   }
+   mode_t mode=parse_perms(t+1);
+   if(mode!=(mode_t)-1)
+      fi->SetMode(mode);
+
+   // link count
+   t = NEXT_TOKEN;
+   if(!t)
+      ERR;
+   fi->SetNlink(atoi(t));
+
+   // user
+   t = NEXT_TOKEN;
+   if(!t)
+      ERR;
+   fi->SetUser(t);
+
+   // group or size
+   char *group_or_size = NEXT_TOKEN;
+
+   // size or month
+   t = NEXT_TOKEN;
+   if(!t)
+      ERR;
+   if(isdigit(*t))
+   {
+      // it's size, so the previous was group:
+      fi->SetGroup(group_or_size);
+      long long size;
+      if(sscanf(t,"%lld",&size)==1)
+	 fi->SetSize(size);
+      t = NEXT_TOKEN;
+      if(!t)
+	 ERR;
+   }
+   else
+   {
+      // it was month, so the previous was size:
+      long long size;
+      if(sscanf(group_or_size,"%lld",&size)==1)
+	 fi->SetSize(size);
+   }
+
+   struct tm date;
+   memset(&date,0,sizeof(date));
+
+   date.tm_mon=parse_month(t);
+   if(date.tm_mon==-1)
+      date.tm_mon=0;
+
+   const char *day_of_month = NEXT_TOKEN;
+   if(!day_of_month)
+      ERR;
+   date.tm_mday=atoi(day_of_month);
+
+   bool year_anomaly=false;
+
+   // time or year
+   t = NEXT_TOKEN;
+   if(!t)
+      ERR;
+   date.tm_hour=date.tm_min=0;
+   int prec=30;
+   if(strlen(t)==5)
+   {
+      sscanf(t,"%2d:%2d",&date.tm_hour,&date.tm_min);
+      date.tm_year=guess_year(date.tm_mon,date.tm_mday,date.tm_hour,date.tm_min) - 1900;
+   }
+   else
+   {
+      if(day_of_month+strlen(day_of_month)+1 == t)
+	 year_anomaly=true;
+      date.tm_year=atoi(t)-1900;
+      /* We don't know the hour.  Set it to something other than 0, or
+       * DST -1 will end up changing the date. */
+      date.tm_hour = 12;
+      prec=12*60*60;
+   }
+
+   date.tm_isdst=-1;
+   date.tm_sec=30;
+
+   fi->SetDate(mktime_from_tz(&date,tz),prec);
+
+   char *name=strtok_r(NULL,"",&next);
+   if(!name)
+      ERR;
+
+   // there are ls which outputs extra space after year.
+   if(year_anomaly && *name==' ')
+      name++;
+
+   if(fi->filetype==fi->SYMLINK)
+   {
+      char *arrow=name;
+      while((arrow=strstr(arrow," -> "))!=0)
+      {
+	 if(arrow!=name && arrow[4]!=0)
+	 {
+	    *arrow=0;
+	    fi->SetSymlink(arrow+4);
+	    break;
+	 }
+	 arrow++;
+      }
+   }
+   fi->SetName(name);
+   fi->SetLongName(line_c);
+
+   return fi;
+}
+
+
 FileInfo::~FileInfo()
 {
    xfree(name);
    xfree(symlink);
    xfree(data);
+   xfree(longname);
 }
 
 int FileSet::Have() const
@@ -721,3 +910,51 @@ void FileSet::SortByPatternList(const char *list_c)
 	    files[i]->SetRank(rank);
    Sort(BYRANK);
 }
+
+void FileInfo::MakeLongName()
+{
+   longname=(char*)xrealloc(longname,80+xstrlen(name)+xstrlen(symlink));
+   char filetype_c='-';
+   switch(filetype)
+   {
+   case NORMAL:	   break;
+   case UNKNOWN:   break;
+   case DIRECTORY: filetype_c='d'; break;
+   case SYMLINK:   filetype_c='l'; break;
+   }
+   int mode1=(defined&MODE?mode:
+      (filetype_c=='d'?0755:(filetype_c=='l'?0777:0644)));
+   sprintf(longname,"%c%s  ",filetype_c,format_perms(mode1));
+   char usergroup[33];
+   usergroup[0]=0;
+   if(defined&(USER|GROUP))
+      sprintf(usergroup,"%.16s%s%.16s",defined&USER?user:"?",
+		  defined&GROUP?"/":"",defined&GROUP?group:"");
+   char size_str[20];
+   strcpy(size_str,"-");
+   if(defined&SIZE)
+      sprintf(size_str,"%lld",(long long)size);
+   int w=20-strlen(usergroup);
+   if(w<1)
+      w=1;
+   sprintf(longname+strlen(longname),"%s %*s ",usergroup,w,size_str);
+   const char *date_str="-";
+   if(defined&DATE)
+      date_str=TimeDate(date).IsoDateTime();
+   sprintf(longname+strlen(longname),"%s %s",date_str,name);
+   if(defined&SYMLINK_DEF)
+      sprintf(longname+strlen(longname)," -> %s",symlink);
+}
+
+int FileSet::EstimateMemory() const
+{
+   int size=sizeof(FileSet)+sizeof(FileInfo*)*fnum;
+   for(int i=0; i<fnum; i++)
+   {
+      size+=sizeof(FileInfo);
+      size+=xstrlen(files[i]->name);
+      size+=xstrlen(files[i]->symlink);
+      size+=xstrlen(files[i]->longname);
+   }
+   return size;
+}
