@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileSet.h,v 1.27 2003/01/27 15:52:46 lav Exp $ */
+/* $Id: FileSet.h,v 1.32 2004/04/22 06:32:55 lav Exp $ */
 
 #ifndef FILESET_H
 #define FILESET_H
@@ -44,6 +44,7 @@ public:
 
    enum	 type
    {
+      UNKNOWN=0,
       DIRECTORY,
       SYMLINK,
       NORMAL
@@ -65,6 +66,7 @@ public:
    };
 
    int rank;
+   char *longname;
 
    ~FileInfo();
    void Init();
@@ -76,6 +78,7 @@ public:
    void SetUser(const char *n);
    void SetGroup(const char *n);
    void LocalFile(const char *name, bool follow_symlinks);
+   static FileInfo *parse_ls_line(const char *line,const char *tz);
 
    void SetMode(mode_t m) { mode=m; defined|=MODE; }
    void SetDate(time_t t,int prec) { date=t; defined|=DATE; date_prec=prec; }
@@ -99,6 +102,9 @@ public:
 
    void SetRank(int r) { rank=r; }
    int GetRank() const { return rank; }
+   void MakeLongName();
+   void SetLongName(const char *s) { xfree(longname); longname=xstrdup(s); }
+   const char *GetLongName() { if(!longname) MakeLongName(); return longname; }
 
    operator const char *() { return name; }
 };
@@ -127,7 +133,7 @@ public:
    FileSet()
    {
       files=files_sort=0;
-      sorted = false;
+      sorted=false;
       fnum=0;
       ind=0;
    }
@@ -143,7 +149,9 @@ public:
    void	 SubtractAny(const FileSet *);
    void  SubtractOlderThan(time_t t);
    void  SubtractDirs();
+   void  SubtractNotDirs();
    void  SubtractNotIn(const FileSet *);
+   void  SubtractSameType(const FileSet *);
    void  SubtractCurr();
    void  Sort(sort_e newsort, bool casefold=false);
    void  Unsort();
@@ -186,6 +194,8 @@ public:
    int Have() const;
 
    FileInfo * operator[](int i) const;
+
+   int EstimateMemory() const;
 };
 
 #endif // FILESET_H
