@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: LocalAccess.cc,v 1.43.2.2 2003/12/22 13:18:51 lav Exp $ */
+/* $Id: LocalAccess.cc,v 1.47 2004/03/25 00:00:05 lav Exp $ */
 
 #include <config.h>
 #include <errno.h>
@@ -296,6 +296,9 @@ int LocalAccess::Do()
       fill_array_info();
       done=true;
       return MOVED;
+   case MP_LIST:
+      SetError(NOT_SUPP);
+      return MOVED;
    }
    return m;
 }
@@ -657,8 +660,7 @@ LocalDirList::LocalDirList(ArgV *a,const char *cwd)
    a->insarg(1,"-l");
    InputFilter *f=new InputFilter(a);
    f->SetCwd(cwd);
-   Delete(buf);
-   buf=new IOBufferFDStream(f,IOBuffer::GET);
+   ubuf=new IOBufferFDStream(f,IOBuffer::GET);
 }
 int LocalDirList::Do()
 {
@@ -671,19 +673,31 @@ int LocalDirList::Do()
       return MOVED;
    }
 
-   if(buf->Error())
+   if(ubuf->Error())
    {
-      SetError(buf->ErrorText());
+      SetError(ubuf->ErrorText());
       return MOVED;
    }
    if(!fg_data)
-      fg_data=buf->GetFgData(false);
-   return STALL;
+      fg_data=ubuf->GetFgData(false);
+   const char *b;
+   int len;
+   ubuf->Get(&b,&len);
+   if(b==0) // eof
+   {
+      buf->PutEOF();
+      return MOVED;
+   }
+   if(len==0)
+      return STALL;
+   buf->Put(b,len);
+   ubuf->Skip(len);
+   return MOVED;
 }
 LocalDirList::~LocalDirList()
 {
-   if(fg_data)
-      delete fg_data;
+   delete fg_data;
+   Delete(ubuf);
 }
 
 #include "modconfig.h"
