@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2001 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2004 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: MirrorJob.cc,v 1.99.2.2 2003/12/17 09:52:52 lav Exp $ */
+/* $Id: MirrorJob.cc,v 1.109 2004/04/22 06:32:55 lav Exp $ */
 
 #include <config.h>
 
@@ -38,6 +38,7 @@
 #include "FindJob.h"
 #include "url.h"
 #include "CopyJob.h"
+#include "pgetJob.h"
 
 void  MirrorJob::PrintStatus(int v,const char *tab)
 {
@@ -51,6 +52,7 @@ void  MirrorJob::PrintStatus(int v,const char *tab)
    case(DONE):
    case(WAITING_FOR_TRANSFER):
    case(TARGET_REMOVE_OLD):
+   case(TARGET_REMOVE_OLD_FIRST):
    case(TARGET_CHMOD):
       break;
 
@@ -117,6 +119,7 @@ void  MirrorJob::ShowRunStatus(StatusLine *s)
    // these have a sub-job
    case(WAITING_FOR_TRANSFER):
    case(TARGET_REMOVE_OLD):
+   case(TARGET_REMOVE_OLD_FIRST):
    case(TARGET_CHMOD):
    case(FINISHING):
    case(DONE):
@@ -184,6 +187,9 @@ void  MirrorJob::HandleFile(FileInfo *file)
       {
 	 bool remove_target=false;
 	 bool cont_this=false;
+	 bool use_pget=(pget_n>1) && target_is_local;
+	 if((file->defined&file->SIZE) && file->size<pget_minchunk*2)
+	    use_pget=false;
 	 if(target_is_local)
 	 {
 	    if(lstat(target_name,&st)!=-1)
@@ -207,7 +213,8 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	    	((file->defined&file->DATE) && (old->defined&old->DATE)
 	    	&& file->date + file->date_prec < old->date - old->date_prec))
 	    && (file->defined&file->SIZE) && (old->defined&old->SIZE)
-	    && file->size >= old->size)
+	    && file->size >= old->size
+	    && !use_pget)
 	    {
 	       cont_this=true;
 	       if(target_is_local && !script_only)
@@ -218,14 +225,17 @@ void  MirrorJob::HandleFile(FileInfo *file)
 		     chmod(target_name,st.st_mode|0200);
 		  }
 	       }
+	       stats.mod_files++;
 	    }
-	    else
+	    else if(!to_rm_mismatched->FindByName(file->name))
 	    {
 	       Report(_("Removing old file `%s'"),
 			dir_file(target_relative_dir,file->name));
 	       remove_target=true;
+	       stats.mod_files++;
 	    }
-	    stats.mod_files++;
+	    else
+	       stats.new_files++;
 	 }
 	 else
 	    stats.new_files++;
@@ -235,14 +245,20 @@ void  MirrorJob::HandleFile(FileInfo *file)
 
 	 if(script)
 	 {
-	    ArgV args("get");
+	    ArgV args(use_pget?"pget":"get");
+	    if(use_pget)
+	    {
+	       args.Append("-n");
+	       args.Append(pget_n);
+	    }
 	    if(cont_this)
 	       args.Append("-c");
 	    if(remove_target)
 	       args.Append("-e");
 	    args.Append(source_session->GetFileURL(file->name));
 	    args.Append("-o");
-	    args.Append(target_session->GetFileURL(file->name));
+	    args.Append(target_is_local?dir_file(target_dir,file->name)
+			:target_session->GetFileURL(file->name));
 	    char *cmd=args.CombineQuoted();
 	    fprintf(script,"%s\n",cmd);
 	    xfree(cmd);
@@ -250,26 +266,45 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	       goto skip;
 	 }
 
-	 FileCopyPeerFA *src_peer=
-	    new FileCopyPeerFA(source_session->Clone(),file->name,FA::RETRIEVE);
-	 FileCopyPeerFA *dst_peer=
-	    new FileCopyPeerFA(target_session->Clone(),file->name,FA::STORE);
+	 if(!use_pget)
+	 {
+	    FileCopyPeerFA *src_peer=
+	       new FileCopyPeerFA(source_session->Clone(),file->name,FA::RETRIEVE);
+	    FileCopyPeerFA *dst_peer=
+	       new FileCopyPeerFA(target_session->Clone(),file->name,FA::STORE);
 
-	 FileCopy *c=FileCopy::New(src_peer,dst_peer,cont_this);
-	 if(remove_source_files)
-	    c->RemoveSourceLater();
-	 if(remove_target)
-	    c->RemoveTargetFirst();
-	 CopyJob *cp=
-	    new CopyJob(c,file->name,"mirror");
-	 if(file->defined&file->DATE)
-	    cp->SetDate(file->date);
-	 if(file->defined&file->SIZE)
-	    cp->SetSize(file->size);
-	 AddWaiting(cp);
-	 transfer_count++;
-	 cp->SetParentFg(this);
-	 cp->cmdline=xasprintf("\\transfer %s",file->name);
+	    FileCopy *c=FileCopy::New(src_peer,dst_peer,cont_this);
+	    if(remove_source_files)
+	       c->RemoveSourceLater();
+	    if(remove_target)
+	       c->RemoveTargetFirst();
+	    CopyJob *cp=
+	       new CopyJob(c,file->name,"mirror");
+	    if(file->defined&file->DATE)
+	       cp->SetDate(file->date);
+	    if(file->defined&file->SIZE)
+	       cp->SetSize(file->size);
+	    AddWaiting(cp);
+	    transfer_count++;
+	    cp->SetParentFg(this);
+	    cp->cmdline=xasprintf("\\transfer %s",file->name);
+	 }
+	 else // pget
+	 {
+	    ArgV *get_args=new ArgV("mirror");
+	    get_args->Append(file->name);
+	    get_args->Append(dir_file(target_dir,file->name));
+	    pgetJob *cp=new pgetJob(source_session->Clone(),get_args);
+	    cp->SetMaxConn(pget_n);
+	    if(remove_source_files)
+	       cp->RemoveSourceLater();
+	    if(remove_target)
+	       cp->RemoveTargetFirst();
+	    AddWaiting(cp);
+	    transfer_count++;
+	    cp->SetParentFg(this);
+	    cp->cmdline=xasprintf("\\transfer %s",file->name);
+	 }
 	 state=WAITING_FOR_TRANSFER;
 	 break;
       }
@@ -327,6 +362,8 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	 mj->verbose_report=verbose_report;
 	 mj->newer_than=newer_than;
 	 mj->parallel=parallel;
+	 mj->pget_n=pget_n;
+	 mj->pget_minchunk=pget_minchunk;
 	 mj->remove_source_files=remove_source_files;
 	 mj->create_target_dir=create_target_dir;
 	 mj->no_target_dir=no_target_dir;
@@ -391,6 +428,8 @@ void  MirrorJob::HandleFile(FileInfo *file)
 #endif /* LSTAT */
 	 break;
       }
+   case FileInfo::UNKNOWN:
+      break;
    }
 skip:
    return;
@@ -413,6 +452,8 @@ void  MirrorJob::InitSets(FileSet *source,FileSet *dest)
       ignore|=FileInfo::IGNORE_DATE_IF_OLDER;
    if(flags&IGNORE_TIME)
       ignore|=FileInfo::DATE;
+   if(flags&IGNORE_SIZE)
+      ignore|=FileInfo::SIZE;
    to_transfer->SubtractSame(dest,ignore);
 
    same->SubtractAny(to_transfer);
@@ -428,6 +469,13 @@ void  MirrorJob::InitSets(FileSet *source,FileSet *dest)
    old_files_set=new FileSet(dest);
    old_files_set->SubtractNotIn(to_transfer);
 
+   to_rm_mismatched=new FileSet(old_files_set);
+   to_rm_mismatched->SubtractSameType(to_transfer);
+   to_rm_mismatched->SubtractNotDirs();
+
+   if(!(flags&DELETE))
+      to_transfer->SubtractAny(to_rm_mismatched);
+
    to_transfer->SortByPatternList(ResMgr::Query("mirror:order",0));
 }
 
@@ -443,8 +491,7 @@ void MirrorJob::HandleChdir(FileAccess * &session, int &redirections)
 	 // cd to another url.
 	 const char *loc_c=session->GetNewLocation();
 	 int max_redirections=ResMgr::Query("xfer:max-redirections",0);
-	 if(loc_c && loc_c[0] && max_redirections>0
-	 && loc_c[strlen(loc_c)-1]=='/')
+	 if(loc_c && max_redirections>0 && last_char(loc_c)=='/')
 	 {
 	    if(++redirections>max_redirections)
 	       goto cd_err_normal;
@@ -555,6 +602,7 @@ int   MirrorJob::Do()
       Roll(source_session);
       state=CHANGING_DIR_SOURCE;
       m=MOVED;
+      /*fallthrough*/
    case(CHANGING_DIR_SOURCE):
       HandleChdir(source_session,source_redirections);
       if(state!=CHANGING_DIR_SOURCE)
@@ -614,6 +662,7 @@ int   MirrorJob::Do()
       target_session->Mkdir(target_dir,(parent_mirror==0));
       state=MAKE_TARGET_DIR;
       m=MOVED;
+      /*fallthrough*/
    case(MAKE_TARGET_DIR):
       res=target_session->Done();
       if(res==FA::IN_PROGRESS)
@@ -626,6 +675,7 @@ int   MirrorJob::Do()
       Roll(target_session);
       state=CHANGING_DIR_TARGET;
       m=MOVED;
+      /*fallthrough*/
    case(CHANGING_DIR_TARGET):
       HandleChdir(target_session,target_redirections);
       if(state!=CHANGING_DIR_TARGET)
@@ -646,7 +696,7 @@ int   MirrorJob::Do()
 	 Delete(target_list_info); target_list_info=0;
 	 return MOVED;
       }
-
+      /*fallthrough*/
    case(GETTING_LIST_INFO):
       HandleListInfo(source_list_info,source_set);
       HandleListInfo(target_list_info,target_set);
@@ -659,11 +709,19 @@ int   MirrorJob::Do()
       stats.dirs++;
 
       InitSets(source_set,target_set);
+      to_rm->Count(&stats.del_dirs,&stats.del_files,&stats.del_symlinks,&stats.del_files);
+      to_rm->rewind();
+      to_rm_mismatched->Count(&stats.del_dirs,&stats.del_files,&stats.del_symlinks,&stats.del_files);
+      to_rm_mismatched->rewind();
+      state=TARGET_REMOVE_OLD_FIRST;
+      return MOVED;
 
+   pre_WAITING_FOR_TRANSFER:
       to_transfer->rewind();
       transfer_count-=root_transfer_count; // leave room for transfers.
       state=WAITING_FOR_TRANSFER;
       m=MOVED;
+      /*fallthrough*/
    case(WAITING_FOR_TRANSFER):
       j=FindDoneAwaitedJob();
       if(j)
@@ -691,25 +749,13 @@ int   MirrorJob::Do()
       break;
 
    pre_TARGET_REMOVE_OLD:
-      to_rm->Count(&stats.del_dirs,&stats.del_files,&stats.del_symlinks,&stats.del_files);
-      to_rm->rewind();
+      if(flags&REMOVE_FIRST)
+	 goto pre_TARGET_CHMOD;
       state=TARGET_REMOVE_OLD;
       m=MOVED;
-
-      if(!(flags&DELETE))
-      {
-	 if(flags&REPORT_NOT_DELETED)
-	 {
-	    for(file=to_rm->curr(); file; file=to_rm->next())
-	    {
-	       Report(_("Old file `%s' is not removed"),
-			dir_file(target_relative_dir,file->name));
-	    }
-	 }
-	 goto pre_TARGET_CHMOD;
-      }
       /*fallthrough*/
    case(TARGET_REMOVE_OLD):
+   case(TARGET_REMOVE_OLD_FIRST):
       j=FindDoneAwaitedJob();
       if(j)
       {
@@ -718,12 +764,34 @@ int   MirrorJob::Do()
 	 transfer_count--;
 	 m=MOVED;
       }
-      while(transfer_count<parallel && state==TARGET_REMOVE_OLD)
+      while(transfer_count<parallel && (state==TARGET_REMOVE_OLD || state==TARGET_REMOVE_OLD_FIRST))
       {
-	 file=to_rm->curr();
+	 file=0;
+	 if(!file && state==TARGET_REMOVE_OLD_FIRST)
+	 {
+	    file=to_rm_mismatched->curr();
+	    to_rm_mismatched->next();
+	 }
+	 if(!file && (state==TARGET_REMOVE_OLD || (flags&REMOVE_FIRST)))
+	 {
+	    file=to_rm->curr();
+	    to_rm->next();
+	 }
 	 if(!file)
-	    goto pre_TARGET_CHMOD;
-	 to_rm->next();
+	    break;
+	 if(!(flags&DELETE))
+	 {
+	    if(flags&REPORT_NOT_DELETED)
+	    {
+	       if(file->defined&file->TYPE && file->filetype==file->DIRECTORY)
+		  Report(_("Old directory `%s' is not removed"),
+			   dir_file(target_relative_dir,file->name));
+	       else
+		  Report(_("Old file `%s' is not removed"),
+			   dir_file(target_relative_dir,file->name));
+	    }
+	    continue;
+	 }
 	 if(script)
 	 {
 	    ArgV args("rm");
@@ -758,6 +826,12 @@ int   MirrorJob::Do()
 		     dir_file(target_relative_dir,file->name));
 	 }
       }
+      if(waiting_num==0)
+      {
+	 if(state==TARGET_REMOVE_OLD)
+	    goto pre_TARGET_CHMOD;
+	 goto pre_WAITING_FOR_TRANSFER;
+      }
       break;
 
    pre_TARGET_CHMOD:
@@ -767,6 +841,7 @@ int   MirrorJob::Do()
       to_transfer->rewind();
       state=TARGET_CHMOD;
       m=MOVED;
+      /*fallthrough*/
    case(TARGET_CHMOD):
       j=FindDoneAwaitedJob();
       if(j)
@@ -837,6 +912,7 @@ int   MirrorJob::Do()
       }
       state=FINISHING;
       m=MOVED;
+      /*fallthrough*/
    case(FINISHING):
       j=FindDoneAwaitedJob();
       if(j)
@@ -852,6 +928,7 @@ int   MirrorJob::Do()
       transfer_count++; // parent mirror will decrement it.
       state=DONE;
       m=MOVED;
+      /*fallthrough*/
    case(DONE):
       break;
    }
@@ -882,7 +959,7 @@ MirrorJob::MirrorJob(MirrorJob *parent,
    source_relative_dir=0;
    target_relative_dir=0;
 
-   to_transfer=to_rm=same=0;
+   to_transfer=to_rm=to_rm_mismatched=same=0;
    source_set=target_set=0;
    new_files_set=old_files_set=0;
    create_target_dir=true;
@@ -907,6 +984,8 @@ MirrorJob::MirrorJob(MirrorJob *parent,
    remove_source_files=false;
 
    parallel=1;
+   pget_n=1;
+   pget_minchunk=0x10000;
 
    source_redirections=0;
    target_redirections=0;
@@ -932,6 +1011,7 @@ MirrorJob::~MirrorJob()
    delete target_set;
    delete to_transfer;
    delete to_rm;
+   delete to_rm_mismatched;
    delete same;
    delete new_files_set;
    delete old_files_set;
@@ -978,8 +1058,18 @@ void MirrorJob::Report(const char *fmt,...)
    va_end(v);
 }
 
+extern "C" {
+#include "getdate.h"
+}
 void MirrorJob::SetNewerThan(const char *f)
 {
+   time_t t=now;
+   t=get_date(f,&t);
+   if(t>0)
+   {
+      newer_than=t;
+      return;
+   }
    struct stat st;
    if(stat(f,&st)==-1)
    {
@@ -1087,10 +1177,14 @@ CMD(mirror)
       {"Remove-source-files",no_argument,0,256+'R'},
       {"parallel",optional_argument,0,'P'},
       {"ignore-time",no_argument,0,256+'i'},
+      {"ignore-size",no_argument,0,257+'i'},
+      {"only-missing",no_argument,0,256+'m'},
       {"log",required_argument,0,256+'s'},
       {"script",    required_argument,0,256+'S'},
       {"just-print",optional_argument,0,256+'S'},
       {"dry-run",   optional_argument,0,256+'S'},
+      {"delete-first",no_argument,0,256+'e'},
+      {"use-pget-n",optional_argument,0,256+'p'},
       {0}
    };
 
@@ -1106,6 +1200,7 @@ CMD(mirror)
    const char *newer_than=0;
    bool  remove_source_files=false;
    int	 parallel=ResMgr::Query("mirror:parallel-transfer-count",0);
+   int	 use_pget=ResMgr::Query("mirror:use-pget-n",0);
    bool	 reverse=false;
    bool	 script_only=false;
    const char *script_file=0;
@@ -1212,12 +1307,24 @@ CMD(mirror)
       case(256+'i'):
 	 flags|=MirrorJob::IGNORE_TIME;
 	 break;
+      case(257+'i'):
+	 flags|=MirrorJob::IGNORE_SIZE;
+	 break;
+      case(256+'m'):
+	 flags|=MirrorJob::IGNORE_TIME|MirrorJob::IGNORE_SIZE;
+	 break;
       case('P'):
 	 if(optarg)
 	    parallel=atoi(optarg);
 	 else
 	    parallel=3;
 	 break;
+      case(256+'p'):
+	 if(optarg)
+	    use_pget=atoi(optarg);
+	 else
+	    use_pget=3;
+	 break;
       case(256+'S'):
 	 script_only=true;
       case(256+'s'):
@@ -1225,6 +1332,9 @@ CMD(mirror)
 	 if(script_file==0)
 	    script_file="-";
 	 break;
+      case(256+'e'):
+	 flags|=MirrorJob::REMOVE_FIRST|MirrorJob::DELETE;
+	 break;
       case('?'):
 	 eprintf(_("Try `help %s' for more information.\n"),args->a0());
       no_job:
@@ -1281,8 +1391,7 @@ CMD(mirror)
 	    }
 	    target_dir=alloca_strdup(target_url.path);
 	 }
-	 if(target_dir[0] && target_dir[strlen(target_dir)-1]=='/'
-	 && basename_ptr(target_dir)[0]!='/')
+	 if(last_char(target_dir)=='/' && basename_ptr(target_dir)[0]!='/')
 	 {
 	    // user wants source dir name appended.
 	    const char *base=basename_ptr(source_dir);
@@ -1335,6 +1444,8 @@ CMD(mirror)
       parallel=64;   // a (in)sane limit.
    if(parallel)
       j->SetParallel(parallel);
+   if(use_pget>1)
+      j->SetPGet(use_pget);
 
    if(script_file)
    {
