@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-1997 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2004 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: MirrorJob.h,v 1.35 2003/02/14 09:32:04 lav Exp $ */
+/* $Id: MirrorJob.h,v 1.39 2004/02/17 10:45:57 lav Exp $ */
 
 #ifndef MIRRORJOB_H
 #define MIRRORJOB_H
@@ -39,6 +39,7 @@ class MirrorJob : public Job
       GETTING_LIST_INFO,
       WAITING_FOR_TRANSFER,
       TARGET_REMOVE_OLD,
+      TARGET_REMOVE_OLD_FIRST,
       TARGET_CHMOD,
       FINISHING,
       DONE
@@ -55,6 +56,7 @@ class MirrorJob : public Job
    FileSet *to_transfer;
    FileSet *same;
    FileSet *to_rm;
+   FileSet *to_rm_mismatched;
    FileSet *old_files_set;
    FileSet *new_files_set;
    void	 InitSets(FileSet *src,FileSet *dst);
@@ -109,6 +111,8 @@ class MirrorJob : public Job
    bool remove_source_files;
 
    int parallel;
+   int pget_n;
+   int pget_minchunk;
 
    mode_t get_mode_mask();
 
@@ -133,7 +137,9 @@ public:
       RETR_SYMLINKS=1<<7,
       NO_UMASK=1<<8,
       ALLOW_CHOWN=1<<9,
-      IGNORE_TIME=1<<10
+      IGNORE_TIME=1<<10,
+      REMOVE_FIRST=1<<11,
+      IGNORE_SIZE=1<<12
    };
 
    void SetFlags(int f,int v)
@@ -170,6 +176,7 @@ public:
    void	 RemoveSourceFiles() { remove_source_files=true; }
 
    void  SetParallel(int p) { parallel=p; }
+   void  SetPGet(int n) { pget_n=n; }
 
    void Fg();
    void Bg();
