@@ -18,13 +18,25 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: NetAccess.cc,v 1.55 2002/03/18 17:16:39 lav Exp $ */
+/* $Id: NetAccess.cc,v 1.68 2004/05/28 07:57:05 lav Exp $ */
 
 #include <config.h>
 
 #include <errno.h>
 #include <assert.h>
 #include <math.h>
+#include <sys/types.h>
+#include <netinet/in.h>
+#ifdef HAVE_NETINET_TCP_H
+# include <netinet/tcp.h>
+#endif
+#include <arpa/inet.h>
+#ifdef HAVE_SYS_IOCTL_H
+# include <sys/ioctl.h>
+#endif
+#ifdef HAVE_TERMIOS_H
+# include <termios.h>
+#endif
 #include "NetAccess.h"
 #include "log.h"
 #include "url.h"
@@ -54,7 +66,7 @@ void NetAccess::Init()
    reconnect_interval_max=300;
    timeout=600;		   // 10 minutes with no events = reconnect
 
-   proxy=0;
+   proxy=proxy_proto=0;
    proxy_port=0;
    proxy_user=proxy_pass=0;
 
@@ -63,6 +75,8 @@ void NetAccess::Init()
    connection_limit=0;	// no limit.
    connection_takeover=false;
 
+   home_auto=xstrdup(FindHomeAuto());
+
    Reconfig(0);
 }
 
@@ -73,6 +87,8 @@ NetAccess::NetAccess()
 NetAccess::NetAccess(const NetAccess *o) : super(o)
 {
    Init();
+   xfree(home_auto);
+   home_auto=xstrdup(o->home_auto);
 }
 NetAccess::~NetAccess()
 {
@@ -81,10 +97,12 @@ NetAccess::~NetAccess()
       delete rate_limit;
    ClearPeer();
 
-   xfree(proxy); proxy=0;
-   xfree(proxy_port); proxy_port=0;
-   xfree(proxy_user); proxy_user=0;
-   xfree(proxy_pass); proxy_pass=0;
+   xfree(proxy);
+   xfree(proxy_port);
+   xfree(proxy_user);
+   xfree(proxy_pass);
+   xfree(proxy_proto);
+   xfree(home_auto);
 }
 
 void NetAccess::Reconfig(const char *name)
@@ -143,7 +161,6 @@ void NetAccess::SetSocketMaxseg(int sock,int socket_maxseg)
       Log::global->Format(1,"setsockopt(TCP_MAXSEG,%d): %s\n",socket_maxseg,strerror(errno));
 #endif
 }
-
 void  NetAccess::SetSocketBuffer(int sock)
 {
    SetSocketBuffer(sock,socket_buffer);
@@ -192,6 +209,45 @@ socklen_t NetAccess::SocketAddrLen(const sockaddr_u *u)
    return sizeof(*u);
 }
 
+#if HAVE_INET_ATON && !HAVE_DECL_INET_ATON
+CDECL int inet_aton(const char *,struct in_addr *);
+#endif
+
+int NetAccess::SocketCreate(int af,int type,int proto)
+{
+   int s=socket(af,type,proto);
+   if(s<0)
+      return s;
+   const char *b=0;
+   sockaddr_u bind_addr;
+   memset(&bind_addr,0,sizeof(bind_addr));
+   bind_addr.in.sin_family=af;
+   if(af==AF_INET)
+   {
+      b=ResMgr::Query("net:socket-bind-ipv4",hostname);
+      if(!(b && b[0] && inet_aton(b,&bind_addr.in.sin_addr)))
+	 b=0;
+   }
+#if INET6
+   else if(af==AF_INET6)
+   {
+      b=ResMgr::Query("net:socket-bind-ipv6",hostname);
+      if(!(b && b[0] && inet_pton(af,b,&bind_addr.in6.sin6_addr)))
+	 b=0;
+   }
+#endif
+   if(b)
+   {
+      int res=bind(s,&bind_addr.sa,sizeof(bind_addr.sa));
+      if(res==-1)
+	 Log::global->Format(0,"**** bind(socket, %s): %s\n",b,strerror(errno));
+   }
+   return s;
+}
+int NetAccess::SocketCreateTCP(int af)
+{
+   return SocketCreate(af,SOCK_STREAM,IPPROTO_TCP);
+}
 int NetAccess::SocketConnect(int fd,const sockaddr_u *u)
 {
    // some systems have wrong connect() prototype, so we have to cast off const.
@@ -202,6 +258,62 @@ int NetAccess::SocketConnect(int fd,const sockaddr_u *u)
    return res;
 }
 
+#ifdef TIOCOUTQ
+static bool TIOCOUTQ_returns_free_space;
+static bool TIOCOUTQ_works;
+static bool TIOCOUTQ_tested;
+static void test_TIOCOUTQ()
+{
+   int sock=socket(PF_INET,SOCK_STREAM,IPPROTO_TCP);
+   if(sock==-1)
+      return;
+   int avail=-1;
+   socklen_t len=sizeof(avail);
+   if(getsockopt(sock,SOL_SOCKET,SO_SNDBUF,(char*)&avail,&len)==-1)
+      avail=-1;
+   int buf=-1;
+   if(ioctl(sock,TIOCOUTQ,&buf)==-1)
+      buf=-1;
+   if(buf>=0 && avail>0 && (buf==0 || buf==avail))
+   {
+      TIOCOUTQ_works=true;
+      TIOCOUTQ_returns_free_space=(buf==avail);
+   }
+   close(sock);
+}
+#endif
+int NetAccess::SocketBuffered(int sock)
+{
+#ifdef TIOCOUTQ
+   if(!TIOCOUTQ_tested)
+      test_TIOCOUTQ();
+   if(!TIOCOUTQ_works)
+      return 0;
+   int buffer=0;
+   if(TIOCOUTQ_returns_free_space)
+   {
+      socklen_t len=sizeof(buffer);
+      if(getsockopt(sock,SOL_SOCKET,SO_SNDBUF,(char*)&buffer,&len)==-1)
+	 return 0;
+      int avail=buffer;
+      if(ioctl(sock,TIOCOUTQ,&avail)==-1)
+	 return 0;
+      if(avail>buffer)
+	 return 0; // something wrong
+      buffer-=avail;
+      buffer=buffer*3/4; // approx...
+   }
+   else
+   {
+      if(ioctl(sock,TIOCOUTQ,&buffer)==-1)
+	 return 0;
+   }
+   return buffer;
+#else
+   return 0;
+#endif
+}
+
 void NetAccess::SayConnectingTo()
 {
    assert(peer_curr<peer_num);
@@ -236,6 +348,7 @@ void NetAccess::SetProxy(const char *px)
    proxy_port=xstrdup(url.port);
    proxy_user=xstrdup(url.user);
    proxy_pass=xstrdup(url.pass);
+   proxy_proto=xstrdup(url.proto);
    ClearPeer();
 }
 
@@ -351,13 +464,16 @@ int NetAccess::Resolve(const char *defp,const char *ser,const char *pr)
 
 // RateLimit class implementation.
 int RateLimit::total_xfer_number;
-RateLimit::BytesPool RateLimit::total;
+RateLimit::BytesPool RateLimit::total[2];
 bool RateLimit::total_reconfig_needed=true;
 
 RateLimit::RateLimit(const char *c)
 {
    if(total_xfer_number==0)
-      total.Reset();
+   {
+      total[GET].Reset();
+      total[PUT].Reset();
+   }
    total_xfer_number++;
    Reconfig(0,c);
 }
@@ -389,22 +505,22 @@ void RateLimit::BytesPool::AdjustTime()
    }
 }
 
-int RateLimit::BytesAllowed()
+int RateLimit::BytesAllowed(dir_t dir)
 {
    if(total_reconfig_needed)
       ReconfigTotal();
 
-   if(one.rate==0 && total.rate==0) // unlimited
+   if(one[dir].rate==0 && total[dir].rate==0) // unlimited
       return LARGE;
 
-   one  .AdjustTime();
-   total.AdjustTime();
+   one  [dir].AdjustTime();
+   total[dir].AdjustTime();
 
    int ret=LARGE;
-   if(total.rate>0)
-      ret=total.pool/total_xfer_number;
-   if(one.rate>0 && ret>one.pool)
-      ret=one.pool;
+   if(total[dir].rate>0)
+      ret=total[dir].pool/total_xfer_number;
+   if(one[dir].rate>0 && ret>one[dir].pool)
+      ret=one[dir].pool;
    return ret;
 }
 
@@ -416,10 +532,10 @@ void RateLimit::BytesPool::Used(int bytes)
       pool-=bytes;
 }
 
-void RateLimit::BytesUsed(int bytes)
+void RateLimit::BytesUsed(int bytes,dir_t dir)
 {
-   total.Used(bytes);
-   one  .Used(bytes);
+   total[dir].Used(bytes);
+   one  [dir].Used(bytes);
 }
 
 void RateLimit::BytesPool::Reset()
@@ -431,18 +547,20 @@ void RateLimit::Reconfig(const char *name,const char *c)
 {
    if(name && strncmp(name,"net:limit-",10))
       return;
-   one.rate     = ResMgr::Query("net:limit-rate",c);
-   one.pool_max = ResMgr::Query("net:limit-max",c);
-   one.Reset(); // to cut bytes_pool.
+   ResMgr::Query("net:limit-rate",c).ToNumberPair(one[GET].rate,one[PUT].rate);
+   ResMgr::Query("net:limit-max",c) .ToNumberPair(one[GET].pool_max,one[PUT].pool_max);
+   one[GET].Reset(); // to cut bytes_pool.
+   one[PUT].Reset();
 
    if(name && !strncmp(name,"net:limit-total-",16))
       total_reconfig_needed=true;
 }
 void RateLimit::ReconfigTotal()
 {
-   total.rate     = ResMgr::Query("net:limit-total-rate",0);
-   total.pool_max = ResMgr::Query("net:limit-total-max",0);
-   total.Reset();
+   ResMgr::Query("net:limit-total-rate",0).ToNumberPair(total[GET].rate,total[PUT].rate);
+   ResMgr::Query("net:limit-total-max",0) .ToNumberPair(total[GET].pool_max,total[PUT].pool_max);
+   total[GET].Reset();
+   total[PUT].Reset();
    total_reconfig_needed = false;
 }
 
@@ -535,6 +653,32 @@ int NetAccess::CountConnections()
    return count;
 }
 
+void NetAccess::PropagateHomeAuto()
+{
+   if(!home_auto)
+      return;
+   for(FA *fo=FirstSameSite(); fo!=0; fo=NextSameSite(fo))
+   {
+      NetAccess *o=(NetAccess*)fo; // we are sure it is NetAccess.
+      if(!o->home_auto)
+      {
+	 o->home_auto=xstrdup(home_auto);
+	 if(!o->home)
+	    o->set_home(home_auto);
+      }
+   }
+}
+const char *NetAccess::FindHomeAuto()
+{
+   for(FA *fo=FirstSameSite(); fo!=0; fo=NextSameSite(fo))
+   {
+      NetAccess *o=(NetAccess*)fo; // we are sure it is NetAccess.
+      if(o->home_auto)
+	 return o->home_auto;
+   }
+   return 0;
+}
+
 
 // GenericParseListInfo implementation
 int GenericParseListInfo::Do()
@@ -547,6 +691,7 @@ int GenericParseListInfo::Do()
    int res;
    int m=STALL;
 
+do_again:
    if(done)
       return m;
 
@@ -557,7 +702,7 @@ int GenericParseListInfo::Do()
       if(use_cache && LsCache::Find(session,"",mode,
 				    &cache_buffer,&cache_buffer_size))
       {
-	 ubuf=new Buffer();
+	 ubuf=new IOBuffer(IOBuffer::GET);
 	 ubuf->Put(cache_buffer,cache_buffer_size);
 	 ubuf->PutEOF();
       }
@@ -569,6 +714,8 @@ int GenericParseListInfo::Do()
 	 ubuf->SetSpeedometer(new Speedometer());
 	 if(LsCache::IsEnabled())
 	    ubuf->Save(LsCache::SizeLimit());
+	 session->Do();
+	 ubuf->Do();
       }
       m=MOVED;
    }
@@ -576,6 +723,17 @@ int GenericParseListInfo::Do()
    {
       if(ubuf->Error())
       {
+	 if(session->GetErrorCode()==FA::NOT_SUPP)
+	 {
+	    if(mode==FA::MP_LIST)
+	    {
+	       mode=FA::LONG_LIST;
+	       delete ubuf;
+	       ubuf=0;
+	       m=MOVED;
+	       goto do_again;
+	    }
+	 }
 	 SetError(ubuf->ErrorText());
 	 Delete(ubuf);
 	 ubuf=0;
@@ -595,13 +753,28 @@ int GenericParseListInfo::Do()
       int old_mode=mode;
 
       FileSet *set=Parse(b,len);
-      if(result)
+      if(set)
       {
-	 result->Merge(set);
-	 delete set;
+	 set->rewind();
+	 for(file=set->curr(); file!=0; file=set->next())
+	 {
+	    // tilde is special.
+	    if(file->name[0]=='~')
+	    {
+	       int len=strlen(file->name);
+	       file->name=(char*)xrealloc(file->name,len+3);
+	       memmove(file->name+2,file->name,len+1);
+	       memcpy(file->name,"./",2);
+	    }
+	 }
+	 if(result)
+	 {
+	    result->Merge(set);
+	    delete set;
+	 }
+	 else
+	    result=set;
       }
-      else
-	 result=set;
 
       Delete(ubuf);
       ubuf=0;
@@ -640,8 +813,8 @@ int GenericParseListInfo::Do()
 	 {
 	    if(file->filetype==file->SYMLINK && follow_symlinks)
 	    {
-	       file->filetype=file->NORMAL;
-	       file->defined &= ~(file->SIZE|file->SYMLINK_DEF|file->MODE|file->DATE);
+	       //file->filetype=file->NORMAL;
+	       file->defined &= ~(file->SIZE|file->SYMLINK_DEF|file->MODE|file->DATE|file->TYPE);
 	       cur->get_size=true;
 	       cur->get_time=true;
 	    }
@@ -718,7 +891,7 @@ GenericParseListInfo::GenericParseListInfo(FileAccess *s,const char *p)
    get_time_for_dirs=true;
    can_get_prec_time=true;
 
-   mode=FA::LONG_LIST;
+   mode=FA::MP_LIST;
 }
 
 GenericParseListInfo::~GenericParseListInfo()
