@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: NetAccess.h,v 1.30 2002/12/25 13:44:31 lav Exp $ */
+/* $Id: NetAccess.h,v 1.34 2004/01/28 09:59:01 lav Exp $ */
 
 #ifndef NETACCESS_H
 #define NETACCESS_H
@@ -54,17 +54,18 @@ protected:
    int	 connection_limit;
    bool	 connection_takeover;
 
+   class RateLimit *rate_limit;
+
    int	 socket_buffer;
    int	 socket_maxseg;
    void	 SetSocketBuffer(int sock);
    void	 SetSocketMaxseg(int sock);
+
    static void KeepAlive(int sock);
    static void ReuseAddress(int sock);
    static void SetSocketBuffer(int sock,int val);
    static void SetSocketMaxseg(int sock,int val);
-
-   class RateLimit *rate_limit;
-
+   static int SocketBuffered(int sock);
    static const char *SocketNumericAddress(const sockaddr_u *u);
    static int SocketPort(const sockaddr_u *u);
    static socklen_t SocketAddrLen(const sockaddr_u *u);
@@ -76,6 +77,11 @@ protected:
    char	 *proxy_port;
    char  *proxy_user;
    char  *proxy_pass;
+   char  *proxy_proto;
+
+   char	 *home_auto;
+   void	 PropagateHomeAuto();
+   const char *FindHomeAuto();
 
    void SayConnectingTo();
 
@@ -157,7 +163,7 @@ protected:
    int get_info_cnt;
 
    int mode;
-   Buffer *ubuf;
+   IOBuffer *ubuf;
 
    bool get_time_for_dirs;
    bool can_get_prec_time;
