@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2002 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2004 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -50,7 +50,7 @@ void TreatFileJob::Begin(const char *d)
 void TreatFileJob::Finish()
 {
    /* next? */
-   char *d=args->getnext();
+   const char *d=args->getnext();
    if(d) {
       /* we have another argument */
       Begin(d);
