@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1998-2002 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1998-2004 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
  */
 
 #include <config.h>
+#include <errno.h>
 #include "buffer.h"
 #include "xmalloc.h"
 #include "FileAccess.h"
@@ -185,11 +186,6 @@ void Buffer::Empty()
       save=true;
 }
 
-int Buffer::Do()
-{
-   return STALL;
-}
-
 Buffer::Buffer()
 {
    error_text=0;
@@ -244,10 +240,217 @@ void Buffer::SetError2(const char *e1,const char *e2)
 }
 #endif
 
+void DirectedBuffer::SetTranslation(const char *enc,bool translit)
+{
+   if(backend_translate)
+      iconv_close(backend_translate);
+   backend_translate=0;
+   if(!enc)
+      return;
+   const char *local_code=ResMgr::Query("file:charset",0);
+   const char *from_code=(mode==GET?enc:local_code);
+   const char *to_code  =(mode==GET?local_code:enc);
+   if(translit)
+   {
+      const char *add="//TRANSLIT";
+      char *tmp_enc=alloca_strdup2(to_code,strlen(add));
+      strcat(tmp_enc,add);
+      to_code=tmp_enc;
+   }
+   backend_translate=iconv_open(to_code,from_code);
+   if(backend_translate==(iconv_t)-1)
+   {
+      Log::global->Format(0,"iconv_open(%s,%s) failed: %s\n",
+			      to_code,from_code,strerror(errno));
+      backend_translate=0;
+   }
+}
+DirectedBuffer::~DirectedBuffer()
+{
+   SetTranslation(0);
+   delete untranslated;
+}
+void DirectedBuffer::ResetTranslation()
+{
+   if(!backend_translate)
+      return;
+   iconv(backend_translate,0,0,0,0);
+   delete untranslated;
+   untranslated=0;
+}
+void DirectedBuffer::PutTranslated(const char *put_buf,int size)
+{
+   if(!backend_translate)
+   {
+      Buffer::Put(put_buf,size);
+      return;
+   }
+   bool from_untranslated=false;
+   if(untranslated && untranslated->Size()>0)
+   {
+      untranslated->Put(put_buf,size);
+      untranslated->Get(&put_buf,&size);
+      from_untranslated=true;
+   }
+   if(size<=0)
+      return;
+   size_t put_size=size;
+
+try_again:
+   if(put_size==0)
+      return;
+   size_t store_size=6*put_size;
+   Allocate(store_size);
+   char *store_buf=buffer+buffer_ptr+in_buffer;
+   store_size=buffer_allocated-(store_buf-buffer);
+   const char *base_buf=put_buf;
+   // do the translation
+   ICONV_CONST char **put_buf_ptr=const_cast<ICONV_CONST char**>(&put_buf);
+   size_t res=iconv(backend_translate,put_buf_ptr,&put_size,&store_buf,&store_size);
+   in_buffer+=store_buf-(buffer+buffer_ptr+in_buffer);
+   if(from_untranslated)
+      untranslated->Skip(put_buf-base_buf);
+   if(res==(size_t)-1)
+   {
+      switch(errno)
+      {
+      case EINVAL: // incomplete character
+	 if(!from_untranslated)
+	 {
+	    if(!untranslated)
+	       untranslated=new Buffer;
+	    untranslated->Put(put_buf,put_size);
+	 }
+	 break;
+      case EILSEQ: // invalid character
+	 Buffer::Put("?");
+	 put_buf++;
+	 put_size--;
+	 goto try_again;
+      case E2BIG:  // no room to store result
+	 if(store_size>=6*put_size)
+	    Allocate(store_size*2);
+	 goto try_again;
+      default:
+	 break;
+      }
+   }
+   return;
+}
+void DirectedBuffer::EmbraceNewData(int len)
+{
+   if(len<=0)
+      return;
+   RateAdd(len);
+   if(backend_translate)
+   {
+      if(!untranslated)
+	 untranslated=new Buffer;
+      // copy the data to free room for translated data
+      untranslated->Put(buffer+buffer_ptr+in_buffer,len);
+      PutTranslated(0,0);
+   }
+   else
+   {
+      in_buffer+=len;
+   }
+   SaveMaxCheck(0);
+}
+
+// IOBufferStacked implementation
+#undef super
+#define super IOBuffer
+int IOBufferStacked::Do()
+{
+   int m=STALL;
+   if(Done() || Error())
+      return m;
+   int res=0;
+   switch(mode)
+   {
+   case PUT:
+      if(down->Broken() && !broken)
+      {
+	 broken=true;
+	 return MOVED;
+      }
+      if(down->Error())
+      {
+	 SetError(down->ErrorText(),down->ErrorFatal());
+	 m=MOVED;
+      }
+      if(in_buffer==0)
+	 return m;
+      res=Put_LL(buffer+buffer_ptr,in_buffer);
+      if(res>0)
+      {
+	 in_buffer-=res;
+	 buffer_ptr+=res;
+	 m=MOVED;
+	 down->Do();
+      }
+      break;
+
+   case GET:
+      m|=down->Do();
+      res=Get_LL(GET_BUFSIZE);
+      if(res>0)
+      {
+	 EmbraceNewData(res);
+	 m=MOVED;
+      }
+      if(eof)
+	 m=MOVED;
+      if(down->Error())
+      {
+	 SetError(down->ErrorText(),down->ErrorFatal());
+	 m=MOVED;
+      }
+      break;
+   }
+   if(res<0)
+      return MOVED;
+   return m;
+}
+int IOBufferStacked::Put_LL(const char *buf,int size)
+{
+   if(down->Broken())
+   {
+      broken=true;
+      return -1;
+   }
+   down->Put(buf,size);
+   return size;
+}
+
+int IOBufferStacked::Get_LL(int)
+{
+   const char *b;
+   int size;
+   down->Get(&b,&size);
+
+   if(!b)
+   {
+      eof=true;
+      return 0;
+   }
+
+   Allocate(size);
+   memcpy(buffer+buffer_ptr+in_buffer,b,size);
+   down->Skip(size);
+   return size;
+}
+
+bool IOBufferStacked::Done()
+{
+   if(super::Done())
+      return down->Done();
+   return false;
+}
+
 // IOBufferFDStream implementation
 #include <fcntl.h>
 #include <unistd.h>
-#include <errno.h>
 #undef super
 #define super IOBuffer
 int IOBufferFDStream::Do()
@@ -278,9 +481,7 @@ int IOBufferFDStream::Do()
       res=Get_LL(GET_BUFSIZE);
       if(res>0)
       {
-	 RateAdd(res);
-	 in_buffer+=res;
-	 SaveMaxCheck(0);
+	 EmbraceNewData(res);
 	 event_time=now;
 	 return MOVED;
       }
@@ -394,7 +595,7 @@ bool IOBufferFDStream::Done()
    return false;
 }
 
-// FileInputBuffer implementation
+// IOBufferFileAccess implementation
 #undef super
 #define super IOBuffer
 IOBufferFileAccess::~IOBufferFileAccess()
@@ -410,9 +611,7 @@ int IOBufferFileAccess::Do()
    int res=Get_LL(GET_BUFSIZE);
    if(res>0)
    {
-      RateAdd(res);
-      in_buffer+=res;
-      SaveMaxCheck(0);
+      EmbraceNewData(res);
       event_time=now;
       return MOVED;
    }
