@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1998-2001 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1998-2004 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: buffer.h,v 1.27.2.1 2003/11/26 08:13:29 lav Exp $ */
+/* $Id: buffer.h,v 1.34 2004/02/17 10:45:58 lav Exp $ */
 
 #ifndef BUFFER_H
 #define BUFFER_H
@@ -30,13 +30,16 @@
 #include "xstring.h"
 
 #include <stdarg.h>
+CDECL_BEGIN
+#include <iconv.h>
+CDECL_END
 
 #define GET_BUFSIZE 0x10000
 #define PUT_LL_MIN  0x2000
 
 class Speedometer;
 
-class Buffer : public SMTask
+class Buffer
 {
 protected:
    char *error_text;
@@ -66,11 +69,7 @@ protected:
 
    void SaveMaxCheck(int addsize);
 
-   virtual ~Buffer();
-
 public:
-   virtual int Do();
-   virtual bool Done() { return in_buffer==0; }
    bool Error() { return error_text!=0; }
    bool ErrorFatal() { return error_fatal; }
    void SetError(const char *e,bool fatal=false);
@@ -121,39 +120,79 @@ public:
 
    void Empty();
 
-   virtual FgData *GetFgData(bool) { return 0; }
-
-   virtual time_t EventTime() { return now; }
-   virtual const char *Status() { return ""; }
-
    Buffer();
+   virtual ~Buffer();
 };
 
-class IOBuffer : public Buffer
+class DirectedBuffer : public Buffer
 {
 public:
    enum dir_t { GET, PUT };
 
 protected:
-   time_t event_time; // used to detect timeouts
+   iconv_t backend_translate;
+   Buffer *untranslated;
    dir_t mode;
+   void EmbraceNewData(int len);
 
 public:
-   IOBuffer(dir_t m)
+   DirectedBuffer(dir_t m) { mode=m; backend_translate=0; untranslated=0; }
+   ~DirectedBuffer();
+   void SetTranslation(const char *be_encoding,bool translit=true);
+   virtual void PutTranslated(const char *buf,int size);
+   void PutTranslated(const char *buf) { PutTranslated(buf,strlen(buf)); }
+   void ResetTranslation();
+   void Put(const char *buf,int size)
+      {
+	 if(mode==PUT && backend_translate)
+	    PutTranslated(buf,size);
+	 else
+	    Buffer::Put(buf,size);
+      }
+   void Put(const char *buf) { Put(buf,strlen(buf)); }
+   dir_t GetDirection() { return mode; }
+};
+
+class IOBuffer : public DirectedBuffer, public SMTask
+{
+protected:
+   time_t event_time; // used to detect timeouts
+
+public:
+   IOBuffer(dir_t m) : DirectedBuffer(m)
       {
 	 event_time=now;
-	 mode=m;
       }
-   time_t EventTime()
+   virtual time_t EventTime()
       {
 	 if(suspended)
 	    return now;
 	 return event_time;
       }
-   bool Done()
+   virtual bool Done()
       {
 	 return(broken || Error() || (eof && (mode==GET || in_buffer==0)));
       }
+   int Do() { return STALL; }
+
+   virtual FgData *GetFgData(bool) { return 0; }
+   virtual const char *Status() { return ""; }
+   virtual int Buffered() { return Size(); }
+};
+
+class IOBufferStacked : public IOBuffer
+{
+   IOBuffer *down;
+
+   int Get_LL(int size);
+   int Put_LL(const char *buf,int size);
+
+public:
+   IOBufferStacked(IOBuffer *b) : IOBuffer(b->GetDirection()) { down=b; }
+   ~IOBufferStacked() { Delete(down); }
+   time_t EventTime() { return down->EventTime(); }
+   int Do();
+   bool Done();
 };
 
 class IOBufferFDStream : public IOBuffer
