@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2002 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2004 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: lftp.cc,v 1.56.2.1 2003/12/17 09:52:56 lav Exp $ */
+/* $Id: lftp.cc,v 1.60 2004/04/16 14:27:36 lav Exp $ */
 
 #include <config.h>
 
@@ -48,6 +48,7 @@
 #include "IdNameCache.h"
 #include "LocalDir.h"
 #include "ConnectionSlot.h"
+#include "misc.h"
 
 #include "confpaths.h"
 
@@ -219,8 +220,7 @@ public:
 
       ::completion_shell=0;
 
-      if(cmd_buf &&
-	    (*cmd_buf==0 || cmd_buf[strlen(cmd_buf)-1]!='\n'))
+      if(cmd_buf && last_char(cmd_buf)!='\n')
       {
 	 exec->Timeout(0);
 	 add_newline=true;
@@ -284,6 +284,9 @@ static void move_to_background()
 	    if(fd!=1 && fd!=2)
 	       close(fd);
 	 }
+	 Log::global->ShowPID();
+	 Log::global->ShowTime();
+	 Log::global->ShowContext();
       }
       close(0);	  // close stdin.
       open("/dev/null",O_RDONLY); // reopen it, just in case.
