@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: mgetJob.h,v 1.7 2002/04/18 10:37:23 lav Exp $ */
+/* $Id: mgetJob.h,v 1.8 2002/07/31 11:28:02 lav Exp $ */
 
 #ifndef MGETJOB_H
 #define MGETJOB_H
@@ -42,7 +42,7 @@ class mgetJob : public GetJob
 
 public:
    int	 Do();
-   void	 PrintStatus(int);
+   void	 PrintStatus(int,const char *);
    void	 ShowRunStatus(StatusLine *s);
 
    mgetJob(FileAccess *session,ArgV *args,bool c,bool md);
