@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: misc.cc,v 1.60 2003/10/03 14:18:47 lav Exp $ */
+/* $Id: misc.cc,v 1.67 2004/06/07 14:22:41 lav Exp $ */
 
 #include <config.h>
 #include "xmalloc.h"
@@ -41,7 +41,9 @@
 # include <sys/time.h>
 #endif
 
+CDECL_BEGIN
 #include <regex.h>
+CDECL_END
 #include "misc.h"
 #include "ProcWait.h"
 #include "SignalHook.h"
@@ -49,7 +51,7 @@
 #include "ResMgr.h"
 #include <mbswidth.h>
 
-#if defined(HAVE_UNSETENV) && !defined(HAVE_UNSETENV_DECL)
+#if HAVE_UNSETENV && !HAVE_DECL_UNSETENV
 CDECL void unsetenv(const char *name);
 #endif
 
@@ -96,7 +98,7 @@ const char *url_file(const char *url,const char *file)
       buf=(char*)xrealloc(buf,buf_size=len);
    if(!url || url[0]==0)
    {
-      strcpy(buf,file);
+      strcpy(buf,file?file:"");
       return buf;
    }
    ParsedURL u(url);
@@ -105,7 +107,10 @@ const char *url_file(const char *url,const char *file)
       strcpy(buf,dir_file(url,file));
       return buf;
    }
-   u.path=(char*)dir_file(u.path,file);
+   if(file && file[0]=='~')
+      u.path=(char*)file;
+   else
+      u.path=(char*)dir_file(u.path,file);
    xfree(buf);
    buf=u.Combine();
    buf_size=xstrlen(buf)+1;
@@ -581,6 +586,8 @@ time_t mktime_from_tz(struct tm *t,const char *tz)
 {
    if(!tz || !*tz)
       return mktime(t);
+   if(!strcasecmp(tz,"GMT"))
+      return mktime_from_utc(t);
    if(isdigit((unsigned char)*tz) || *tz=='+' || *tz=='-')
    {
       char *tz1=string_alloca(strlen(tz)+4);
@@ -789,12 +796,16 @@ char *xasprintf(const char *format, ...)
  * file/name -> "file"
  * file/name/ -> "file"
  * file -> ""
- * note: last differs from dirname (wwould return ".")
+ * note: the last differs from dirname(1) (which would return ".")
  *
  */
 char *dirname_alloc(const char *fn)
 {
    char *ret=xstrdup(fn);
+   return dirname_modify(ret);
+}
+char *dirname_modify(char *ret)
+{
    int len=strlen(ret);
 
    /* remove trailing slashes */
@@ -810,3 +821,9 @@ char *dirname_alloc(const char *fn)
       *slash=0;
    return ret;
 }
+
+char last_char(const char *str)
+{
+   int len=strlen(str);
+   return str[len-(len>0)];
+}
