@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1999-2001 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1999-2004 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: resource.cc,v 1.86 2003/11/11 07:27:25 lav Exp $ */
+/* $Id: resource.cc,v 1.104 2004/05/26 12:31:43 lav Exp $ */
 
 #include <config.h>
 
@@ -170,12 +170,17 @@ static ResDecl
    ResDecl01 ("ftp:anon-user",		  "anonymous",0,0),
    ResDecl01a("ftp:auto-sync-mode",	  "",	   ResMgr::ERegExpValidate,0),
    ResDecl01b("ftp:bind-data-socket",	  "yes",   ResMgr::BoolValidate,0),
+   ResDecl01f("ftp:charset",		  "",	   ResMgr::CharsetValidate,0),
+   ResDecl01g("ftp:client",		  PACKAGE"/"VERSION,0,0),
    ResDecl01e("ftp:device-prefix",	  "no",	   ResMgr::BoolValidate,0),
    ResDecl01c("ftp:fix-pasv-address",	  "yes",   ResMgr::BoolValidate,0),
    ResDecl01d("ftp:fxp-force",		  "no",	   ResMgr::BoolValidate,0),
    ResDecl02 ("ftp:fxp-passive-source",	  "no",	   ResMgr::BoolValidate,ResMgr::NoClosure),
+   ResDecl02d("ftp:fxp-passive-sscn",	  "yes",   ResMgr::BoolValidate,ResMgr::NoClosure),
    ResDecl02b("ftp:home",		  "",	   0,0),
    ResDecl02a("ftp:site-group",		  "",	   0,0),
+   ResDecl02c("ftp:lang",		  "",	   0,0),
+   ResDecl03a("ftp:list-empty-ok",	  "no",	   0,0),
    ResDecl03 ("ftp:list-options",	  "",	   0,0),
    ResDecl04 ("ftp:nop-interval",	  "120",   ResMgr::UNumberValidate,0),
    ResDecl05 ("ftp:passive-mode",	  "on",    ResMgr::BoolValidate,0),
@@ -191,6 +196,7 @@ static ResDecl
    ResDecl11a("ftp:ssl-allow",		  "yes",   ResMgr::BoolValidate,0),
    ResDecl11b("ftp:ssl-force",		  "no",	   ResMgr::BoolValidate,0),
    ResDecl11c("ftp:ssl-protect-data",	  "no",	   ResMgr::BoolValidate,0),
+   ResDecl11g("ftp:ssl-protect-fxp",	  "no",    ResMgr::BoolValidate,0),
    ResDecl11f("ftp:ssl-protect-list",	  "yes",   ResMgr::BoolValidate,0),
    ResDecl11d("ftp:ssl-auth",		  "TLS",   AuthArgValidate,0),
    ResDecl11e("ftp:ssl-allow-anonymous",  "no",	   ResMgr::BoolValidate,0),
@@ -198,11 +204,16 @@ static ResDecl
    ResDecl12 ("ftp:stat-interval",	  "1",	   ResMgr::UNumberValidate,0),
    ResDecl13 ("ftp:sync-mode",		  "on",    ResMgr::BoolValidate,0),
    ResDecl14 ("ftp:use-abor",		  "yes",   ResMgr::BoolValidate,0),
+   ResDecl14a("ftp:use-feat",		  "yes",   ResMgr::BoolValidate,0),
    ResDecl15 ("ftp:use-fxp",		  "yes",   ResMgr::BoolValidate,0),
+   ResDecl15e("ftp:use-hftp",		  "yes",   ResMgr::BoolValidate,0),
    ResDecl15b("ftp:use-mdtm",		  "yes",   ResMgr::BoolValidate,0),
+   ResDecl15d("ftp:use-mdtm-overloaded",  "no",	   ResMgr::BoolValidate,0),
+   ResDecl15f("ftp:use-mlsd",		  "no",    ResMgr::BoolValidate,0),
    ResDecl15c("ftp:use-pret",		  "yes",   ResMgr::BoolValidate,0),
    ResDecl15a("ftp:use-site-chmod",	  "yes",   ResMgr::BoolValidate,0),
    ResDecl16a("ftp:use-site-idle",	  "no",    ResMgr::BoolValidate,0),
+   ResDecl16d("ftp:use-site-utime",	  "no",	   ResMgr::BoolValidate,0),
    ResDecl16b("ftp:use-size",		  "yes",   ResMgr::BoolValidate,0),
    ResDecl16 ("ftp:use-stat",		  "yes",   ResMgr::BoolValidate,0),
    ResDecl16c("ftp:use-telnet-iac",	  "yes",   ResMgr::BoolValidate,0),
@@ -258,7 +269,19 @@ static ResDecl
    ResDecl46 ("mirror:order",		  "*.sfv *.sig *.md5* *.sum * */", 0,ResMgr::NoClosure),
    ResDecl47 ("mirror:parallel-directories", "yes", ResMgr::BoolValidate,ResMgr::NoClosure),
    ResDecl48 ("mirror:parallel-transfer-count", "1", ResMgr::UNumberValidate,ResMgr::NoClosure),
-   ResDecl49 ("mirror:exclude-regex",	  "(^|/)(\\.in\\.|\\.nfs)",ResMgr::ERegExpValidate,ResMgr::NoClosure);
+   ResDecl49 ("mirror:exclude-regex",	  "(^|/)(\\.in\\.|\\.nfs)",ResMgr::ERegExpValidate,ResMgr::NoClosure),
+   ResDecl50 ("mirror:use-pget-n",	  "1",	   ResMgr::UNumberValidate,ResMgr::NoClosure);
+
+static ResDecl
+   res_sftp_packets  ("sftp:max-packets-in-flight","8",ResMgr::UNumberValidate,0),
+   res_sftp_proto_ver("sftp:protocol-version","4",ResMgr::UNumberValidate,0),
+   res_sftp_rsize    ("sftp:size-read", "0x8000",ResMgr::UNumberValidate,0),
+   res_sftp_wsize    ("sftp:size-write","0x8000",ResMgr::UNumberValidate,0),
+   res_sftp_conn_prog("sftp:connect-program","ssh -ax",0,0),
+   res_sftp_srv_prog ("sftp:server-program","sftp",0,0);
+
+static ResDecl
+   res_file_charset  ("file:charset",  "",   ResMgr::CharsetValidate,ResMgr::NoClosure);
 
 #ifdef USE_SSL
 static ResDecl
@@ -288,11 +311,15 @@ static ResDecl
 
 static ResDecl
    fish_shell  ("fish:shell", "/bin/sh",  0,0),
-   fish_connect("fish:connect-program", "ssh", 0,0);
+   fish_connect("fish:connect-program", "ssh -ax", 0,0);
 
 static ResDecl
    res_dir_colors ("color:dir-colors",   "",   0,ResMgr::NoClosure);
 
+#ifdef HAVE_LANGINFO_H
+# include <langinfo.h>
+#endif
+
 void ResMgr::ClassInit()
 {
    for(ResDecl *scan=type_chain; scan; scan=scan->next)
@@ -356,4 +383,10 @@ void ResMgr::ClassInit()
       dir_colors=getenv("ZLS_COLORS"); /* zsh */
    if(dir_colors)
       Set("color:dir-colors",0,dir_colors);
+
+#if defined(HAVE_NL_LANGINFO) && defined(CODESET)
+   char *cs=nl_langinfo(CODESET);
+   if(cs)
+      Set("file:charset",0,cs);
+#endif
 }
