@@ -25,6 +25,7 @@
 #include "url.h"
 #include "ascii_ctype.h"
 #include "ConnectionSlot.h"
+#include "bookmark.h"
 #include "misc.h"
 
 /*
@@ -33,6 +34,7 @@
 */
 
 static bool valid_slot(const char *s);
+static bool valid_bm(const char *s);
 
 ParsedURL::ParsedURL(const char *url,bool proto_required,bool use_rfc1738)
 {
@@ -76,9 +78,11 @@ ParsedURL::ParsedURL(const char *url,bool proto_required,bool use_rfc1738)
       path=scan+10;
       goto decode;
    }
-   else if(scan[0]==':' && !strncmp(base,"slot:",5) && valid_slot(scan+1))
+   else if(scan[0]==':'
+   && ((!strncmp(base,"slot:",5) && valid_slot(scan+1))
+       || (!strncmp(base,"bm:",3) && valid_bm(scan+1))))
    {
-      // special form for selecting a connection slot
+      // special form for selecting a connection slot or a bookmark
       *scan++=0;
       proto=base;
       host=scan;
@@ -202,13 +206,14 @@ decode:
    url::decode_string(host);
    url::decode_string(path);
 
+   FileAccess *fa=0;
    if(!xstrcmp(proto,"slot"))
    {
-      xfree(orig_url);
-      orig_url=0;
-      FileAccess *fa=ConnectionSlot::FindSession(host);
+      fa=ConnectionSlot::FindSession(host);
       if(!fa)
 	 return;
+      xfree(orig_url);
+      orig_url=0;
       char *orig_path=alloca_strdup(path);
       xfree(memory);
       memory=(char*)xmalloc(
@@ -257,6 +262,19 @@ decode:
 	    strcat(path,"/");
       }
    }
+   else if(!xstrcmp(proto,"bm"))
+   {
+      const char *bm=lftp_bookmarks.Lookup(host);
+      if(!bm)
+	 return;
+      ParsedURL bu(url_file(bm,path+(path && path[0]=='/')));
+      // move the data
+      xfree(memory);
+      char *old_orig_url=orig_url;
+      memcpy(this,&bu,sizeof(bu));
+      bu.memory=0; // so that dtor won't free it
+      orig_url=old_orig_url;
+   }
 }
 
 static bool valid_slot(const char *cs)
@@ -265,15 +283,28 @@ static bool valid_slot(const char *cs)
    char *slash=strchr(s,'/');
    if(slash)
       *slash=0;
+   url::decode_string(s);
    return 0!=ConnectionSlot::Find(s);
 }
+static bool valid_bm(const char *bm)
+{
+   char *s=alloca_strdup(bm);
+   char *slash=strchr(s,'/');
+   if(slash)
+      *slash=0;
+   url::decode_string(s);
+   const char *url=lftp_bookmarks.Lookup(s);
+   return(url && !strchr(url,' ') && !strchr(url,'\t'));
+}
 
 int url::path_index(const char *base)
 {
    const char *scan=base;
    while(is_ascii_alpha(*scan))
       scan++;
-   if(scan[0]==':' && scan[1]=='/' && scan[2]=='/')
+   if(scan[0]!=':')
+      return 0;
+   if(scan[1]=='/' && scan[2]=='/')
    {
       // found protocol
       const char *slash=strchr(scan+3,'/');
@@ -281,11 +312,19 @@ int url::path_index(const char *base)
 	 return slash-base;
       return strlen(base);
    }
-   else if(scan[0]==':' && !strncmp(base,"file:",5))
+   else if(!strncmp(base,"file:",5))
    {
       // special form for file protocol
       return scan+1-base;
    }
+   else if((!strncmp(base,"slot:",5) && valid_slot(base+5))
+	|| (!strncmp(base,"bm:",3) && valid_bm(base+3)))
+   {
+      const char *slash=strchr(scan+1,'/');
+      if(slash)
+	 return slash-base;
+      return strlen(base);
+   }
    return 0;
 }
 
